@@ -24,12 +24,9 @@ check() {
 }
 
 check master 
+check b2.24
 check b2.23
-check b2.22
-check b2.21
-check b2.20
-check b2.19
-check wosch 
+check $(logname) 
 
 echo "Status: $RET"
 ls -l $logdir/log.*
