@@ -9,6 +9,8 @@ Components.utils.import("resource://indexdata/util/logging.js");
 var logger = logging.getLogger('');
 var contentFilter = Components.classes["@indexdata.com/contentfilter;1"]
                      .getService().wrappedJSObject;
+var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"]
+         .getService(Components.interfaces.nsIWindowMediator);
 var Connector = function (template) {
   //hash of tasks indexed by name
   this.template = template || null;
@@ -18,7 +20,9 @@ var Connector = function (template) {
   this.taskList = [];
   //tasks by id
   this.tasksById = {};
+  //dom of the scraped page
   this.forcedPageWindow = null;
+  this.forcedBrowser = null;
   //implementation of nsIWebProgress
   this.taskArgumentsUpdateCb = null;
   this.stepParametersUpdateCb = null;
@@ -35,17 +39,29 @@ var Connector = function (template) {
 };
 
 Connector.prototype = {
+  //TODELETE
+  getBrowser: function () {
+    dump(XPCNativeWrapper.unwrap(this.forcedBrowser));
+    if (this.forcedBrowser) return this.forcedBrowser.wrappedJSObject;
+    return wm.getMostRecentWindow("navigator:browser").gBrowser;
+  },
+  //TODELETE
+  setBrowser: function (browser) {
+    this.forcedBrowser = browser;
+  },
+
   // page accessibility
   getPageWindow: function () {
     var pageWindow = null;
     if (this.forcedPageWindow) {
-      //set page window takes precedence
+      //set page window takes precedence (e.g engine)
       pageWindow = this.forcedPageWindow;
     } else {
-      var browser = core.getBrowser();
-      pageWindow = browser.contentWindow;
+      // try to get the current tab
+      var mainWindow = wm.getMostRecentWindow("navigator:browser");
+      if (mainWindow) pageWindow = mainWindow.gBrowser;
     }
-    if (pageWindow && (pageWindow.document))
+    if (pageWindow && (pageWindow.contentDocument || pageWindow.document))
       return pageWindow;
     else
       throw new Error('The page window is null or malformed (no content doc)');
@@ -56,7 +72,13 @@ Connector.prototype = {
   },
   
   getRealPageDoc: function () {
-    return this.getPageWindow().document;
+    var pW = this.getPageWindow();
+    if (pW.contentDocument)
+      return pW.contentDocument;
+    else if (pW.document)
+      return pW.document;
+    else
+      return null;
   },
   
   getPageDoc: function () {
