@@ -1,5 +1,4 @@
 var EXPORTED_SYMBOLS = ["NavTo"];
-
 Components.utils.import('resource://indexdata/runtime/core.js');
 Components.utils.import('resource://indexdata/runtime/Step.js');
 Components.utils.import('resource://indexdata/runtime/StepError.js');
@@ -157,7 +156,8 @@ NavTo.prototype.getRawXml = function(task) {
     var eventhandler = function () { // This gets called once we have a blank page
         task.debug("event: " + event );
         try {
-            core.getBrowser().removeEventListener(event, eventhandler, false);
+            task.connector.getPageWindow().
+                removeEventListener(event, eventhandler, false);
             var doc = task.connector.getRealPageDoc();
             doc.body.innerHTML = "Loading raw XML from " +
             "<a href=\"" + url +"\" >" + url + "</a> <p/>\n";
@@ -196,7 +196,8 @@ NavTo.prototype.getRawXml = function(task) {
         return false;
     }
     task.debug("About to blank the page");
-    core.getBrowser().addEventListener(event, eventhandler, false);
+    task.connector.getPageWindow().
+        addEventListener(event, eventhandler, false);
     task.interrupt(30000,interruptcallback);  // 30 secs
     this.getPageDoc().location = "about:blank";
 };
@@ -216,7 +217,7 @@ NavTo.prototype.run = function (task) {
     doc.location.hash = url;
   } else {
     task.info("Changing location to " + url, this);
-    core.getBrowser().loadURI(url);
+    doc.location = url;
   }
 };
 
