@@ -315,6 +315,20 @@ Xpattern.prototype.isRepeatable = function() {
   return this.repeatable;
 };
 
+// Check if we have a plain node, just a tag name
+Xpattern.prototype.isPlainNode = function() {
+  return this.isPlainVarNode() && ! this.variable;
+};
+
+// Check for a plain node that may have a $variable
+Xpattern.prototype.isPlainVarNode = function() {
+  if ( this.firstChild || this.attributes ||
+       this.negation || this.modifiers )
+    return false;
+  return true;
+};
+
+
 Xpattern.prototype.getStringoffset = function() {
     return this.stringoffset;
 };
