@@ -35,7 +35,7 @@ Components.utils.import('resource://indexdata/util/xmlHelper.js');
 Components.utils.import('resource://indexdata/util/logging.js');
 var logger = logging.getLogger();
 var debug = false;  // Produces a very detailed dump of the process
-debug = true;
+//debug = true;
 
 
 // constructor. Takes a Dom node that indicates the area to start from
@@ -69,7 +69,7 @@ XpatternMaker.prototype.addGroup =
     function (node, variable, cardinality, attrs, grouptype, parent, groupnum ) {
       if (typeof(groupnum) == "undefined" )  
         groupnum = this.nextgroupnumber++;
-      grp = { "node": node,
+      var grp = { "node": node,
               "variable": variable,
               "cardinality": cardinality,
               "attr" : attrs,
@@ -205,7 +205,7 @@ XpatternMaker.prototype._grouppattern = function (groupno) {
 // Check if a single node matches a target dom node
 XpatternMaker.prototype._nodematch = function (n) {
     var tg = null;
-    for ( targ in this.targets ) {
+    for ( var targ in this.targets ) {
         if ( this.targets[targ].node === n ) { 
             tg = this.targets[targ];
         }
@@ -309,7 +309,6 @@ XpatternMaker.prototype._fixgroups = function(patt) {
   dummyroot.setType("DummyRoot");
   var n = this.groups.length-1;
   while( n > 0 ) {
-    //this._fixonegroup(dummyroot,n);   // ### OLD
     var ok = this._fixonegroup(patt, dummyroot, null, n);
     if (debug)
       logger.debug("After fixing group " + n + " ok=" + ok + " " +
@@ -318,6 +317,18 @@ XpatternMaker.prototype._fixgroups = function(patt) {
         return null; // impossible combination
     n--;
   }
+  // Clean up the pattern.
+  // Make some TR nodes repeatable
+  this._fixRepeatedTr(dummyroot.firstChild);
+  
+  // Trim surrounding nodes that could be part of an or-bag
+  // f.ex the first br in A : BR : ( B $b | U $u | BR )+ : BR
+  n = 0;
+  while(  this._mergeOrBagPrecedents(dummyroot.firstChild, dummyroot, null) ) {
+    n++;  // try again if we found something to merge, there might more than one
+    logger.debug("After merging or-bag precedent " + n + " " + dummyroot.dumpString(-1) );
+    // (note, this loop only writes debug, the work is done in the condition)
+  }
   return dummyroot.firstChild;
 };
 
@@ -436,13 +447,6 @@ XpatternMaker.prototype._makegroup = function(xp, parent, prev, groupno) {
     logger.debug("xpmk_mk: Check for " + group.dumpString(-1) +
       " returns " + ok );
   // convert into a OR-bag if that's what we need
-    // TODO - Check if we can skip some nodes, for example
-    //  ( U $x | U )+
-    // The second U above can be removed, because
-    //  - or-bags repeat
-    //  - the first U has no conditions (siblings, attrs, patterns)
-    // Actually, the deduplicating logic below is a good start,
-    // probably it can be refined.
   var seen = {}; // deduplicate
   if ( ok && this.groups[groupno].grouptype == "|" ) { 
     var ornode = new Xpattern();  // the or-node inside the group
@@ -452,29 +456,174 @@ XpatternMaker.prototype._makegroup = function(xp, parent, prev, groupno) {
     // Turn all children into alternatives
     var child = ornode.firstChild;
     var prev = null;
+    var tail = [];
     while ( child ) {
       var nodestr = child.dumpString(-2);  // this node only
       // Can't use -1 here, it dumps child's nextSibling chain too
       if ( child.firstChild )
-        nodestr += " " + child.dumpString(-1); // and all children
+        nodestr += " { " + child.firstChild.dumpString(-1) + " }"; // and all children
       if(debug) 
-        logger.debug("xpmk_mk_or: '" + nodestr + "'");
+        logger.debug("xpmk_mk_or: '" + nodestr + "' " +
+           "p=" + child.isPlainNode() + " pv=" + child.isPlainVarNode() );
       if ( !seen[nodestr] ) {
-        child.nextAlternative = child.nextSibling;
-        child.nextSibling = null;
-        prev = child;
-        child = child.nextAlternative;
-        seen[nodestr] = 1;
+        if ( ! child.isPlainNode() ) {  // regular pattern node
+          child.nextAlternative = child.nextSibling;
+          child.nextSibling = null;
+          prev = child;
+          child = child.nextAlternative;
+          seen[nodestr] = 1;
+          var plain = nodestr.replace ( /\$\w+\s*/, "" ); // remove first $var
+          logger.debug("xpmk_mk_or:   plain: '" + plain + "'" );
+          seen[plain] = 1;
+        } else { // a plain node, collect into tail
+          logger.debug("xpmk_mk_or:  collected tail '" + nodestr + "'" );
+          tail.push(child);
+          seen[nodestr] = 2;
+          tailend = child;
+          prev.nextAlternative = child.nextSibling;
+          child = child.nextSibling;
+        }
       } else { // seen it before, skip it
         prev.nextAlternative = child.nextSibling;
         child = child.nextSibling;
+        logger.debug("xpmk_mk_or:   skipping '" + nodestr + "': " + seen[nodestr] );
       }
     }
+    for ( var i=0; i<tail.length; i++){
+      tail[i].nextSibling = null;
+      logger.debug("xpmk_mk_or:  Appending tail " + tail[i].dumpString(-1) );
+      prev.nextAlternative = tail[i];
+      prev = prev.nextAlternative;
+      prev.nextAlternative = null;
+    }
   }
   return ok;
 }; // _makegroup
 
+// Recurse through the pattern, looking for or-bags that can are preceded
+// or followed by things that are already included in the or-bag. Remove
+// such nodes, the or-bag will handle them. For example the first and
+// last BR in
+//   A : BR : ( U $u | B $b | BR ) : BR
+// Returns true if something removed, in which case it should be called again
 
+XpatternMaker.prototype._mergeOrBagPrecedents = function(node, parent, prev) {
+  while (node) {
+    if ( this._mergeCandidate(node) ) {
+      if ( parent ) {
+        parent.firstChild = node.nextSibling;
+        logger.debug("xpmk_merge: Merged firstChild " + parent.dumpString(-1) );
+      } else {
+        prev.nextSibling = node.nextSibling;
+        logger.debug("xpmk_merge: Merged nextSibling " + prev.dumpString(-1) );
+      }
+      return true;
+    } else if (this._tailmergeCandidate(node)) {
+      node.nextSibling = node.nextSibling.nextSibling;
+      logger.debug("xpmk_merge: Tail merge " + node.dumpString(-1) );
+    } else { // not removed, iterate further
+      if ( this._mergeOrBagPrecedents(node.firstChild, node, null) )
+        return true;
+      prev = node;
+      parent = null;
+      node = node.nextSibling;
+    } // not removed
+  } // node loop
+  return false;
+}; // _mergeOrBagPrecedents
+
+// Check if one node is already contained in the following ornode
+XpatternMaker.prototype._mergeCandidate = function(node) {
+  if ( ! node.isPlainNode() )
+    return false; // only plain nodes count
+  if ( ! ( node.nextSibling && node.nextSibling.nodeType == "(" ) )
+    return false; // not followed by a group node
+  var ornode = node.nextSibling.firstChild;
+  if ( ! ( ornode && ornode.nodeType == "|" ) )
+    return false; // not an or-bag
+  var alt = ornode.firstChild;
+  while (alt) {
+    if ( alt.nodeType == node.nodeType ) {
+      if ( ! alt.isPlainNode() )
+        return false; // disregard alts with variables, etc
+      return true; // can not find any more objections
+    }
+    alt = alt.nextAlternative;
+  }
+  return false; // didn't find anything similar
+}
+
+// Check if we have an or-bag that is followed by a node
+// that is also contained in the or-bag
+XpatternMaker.prototype._tailmergeCandidate = function(node) {
+  if ( node.nodeType != "(" )
+    return false;
+  var ornode = node.firstChild;
+  if ( ! ( ornode && ornode.nodeType == "|" ) )
+    return false; // not an or-bag, must be a regular group
+  if ( ! node.nextSibling )
+    return false; // nothing after it
+  var alt = ornode.firstChild;
+  while (alt) {
+    if ( alt.nodeType == node.nextSibling.nodeType ) {
+      if ( ! alt.isPlainNode() )
+        return false; // disregard alts with variables, etc
+      return true; // can not find any more objections
+    }
+    alt = alt.nextAlternative;
+  }
+  return false;
+}
+
+// Helper to check if a node contains only repeatable children
+// in those nodes that the user has explicitly defined
+XpatternMaker.prototype._allrepeatable = function(n) {
+  while ( n ) {
+    if ( n.nodeType == "(" || n.nodeType == "|" ) { // group node
+      if ( ! n.repeatable )
+        return false; // if a group is repeatable, don't look at children
+    } else { // regular node
+      if ( n.variable && ! n.repeatable )
+        return false;
+      if ( !this._allrepeatable(n.firstChild) )
+        return false;
+    }
+    n = n.nextSibling;
+  }
+  return true;
+}
+
+// Recurse through the pattern, and make some TR nodes
+// repeatable. Only if
+//  - all children of the TR are repeatable
+//  - the TR is not followed by another TR 
+XpatternMaker.prototype._fixRepeatedTr = function(node) {
+  var ctx = this;
+  node.foreachnode( function(n) {
+    if ( n.nodeType.toUpperCase() != "TR" )
+      return;
+    if ( n.repeatable )
+      return; // already repeating, no point in checking
+    if ( ! n.firstChild )
+      return; // a plain empty TR
+    if ( ! ctx._allrepeatable(n.firstChild) )
+      return; // non-repeatable user-defined nodes somewhere
+    var nxt = n.nextSibling;
+    if ( nxt && nxt.nodeType.toUpperCase() == "TR" &&
+          !nxt.isPlainNode() ) 
+      return; // pattern looks in the next TR already
+    // If we get so far, we have a TR that should be repeatable
+    n.repeatable = true;
+    logger.debug("xpmk_trfix: Found a TR that should repeat!");
+    // remove following simple TR nodes from pattern
+    while ( n.nextSibling && 
+            n.nextSibling.nodeType.toUpperCase() == "TR" &&
+            n.nextSibling.isPlainNode() ) {
+      n.nextSibling = n.nextSibling.nextSibling;
+      logger.debug("xpmk_trfix: Removed following plain TR");
+    }
+  });
+}
 
 ///////////////////////////////
 // Unit tests
@@ -489,7 +638,7 @@ XpatternMaker.prototype.unitTest = function ( ) {
       "<U>Author</U><br/>" +
       "<B>Publisher</B><br/>" +
       "<I>Description</I><br/>" +
-      "</div>"+
+    "</div>"+
     "<div>" +    // div 2 - table 
       "<A href='http://indexdata.com'>Title</A><br/>"+
       "<TABLE>"+
@@ -512,6 +661,21 @@ XpatternMaker.prototype.unitTest = function ( ) {
       "<U>Second Author</U>" + 
       "<B>Publisher</B>" +
     "</div>"+
+    "<div>" + // div 4 - another test for CP-3412, optimized or-bags
+      "<TR>" +
+      "<TD><U>First Author </U></TD>" +
+      "<TD><U>Second Author</U></TD>" +
+      "<TD><B>Publisher</B></TD>" +
+      "</TR>" +
+    "</div>"+
+    "<div>" + // div 5 - for the TR fixes
+      "<I>Title</I>" +
+      "<table>" +
+      "<TR><TD><U>First Author </U></TD></TR>" +
+      "<TR><TD><U>Second Author</U></TD></TR>" +
+      "<TR><TD><B>Publisher</B></TD></TR>" +
+      "</table>" +
+    "</div>"+
     "</body>";
 
   const tests = [
@@ -723,7 +887,7 @@ XpatternMaker.prototype.unitTest = function ( ) {
         { p:"//body/div[2]/TABLE/TR/TD[2]", group:1, v: "publisher" },
         { p:"//body/div[2]/TABLE/TR/TD[3]", group:1, v: "description" },
       ],
-      ex: "A $title : br : TABLE  { TR  { ( TD $author  | TD $publisher  | TD $description ) +} }",
+      ex: "A $title : br : TABLE  { TR + { ( TD $author  | TD $publisher  | TD $description ) +} }",
     },
     { name: "Orbag-3",
       // Check that the repeated BR tags only come once in the list
@@ -733,7 +897,8 @@ XpatternMaker.prototype.unitTest = function ( ) {
         { p:"//body/div[1]/A", v: "title", group:1 },
         { p:"//body/div[1]/B", v: "publisher", group:1 }
       ],
-      ex: "( A $title  | br  | U  | B $publisher ) +",
+      //ex: "( A $title  | br  | U  | B $publisher ) +",
+      ex: "( A $title | B $publisher | br | U ) +",
     },
     { name: "Orbag-4",
       // Match the table rows instead
@@ -768,19 +933,81 @@ XpatternMaker.prototype.unitTest = function ( ) {
         { p:"//body/div[3]/B", group:1, v: "publisher" },
       ],
       ex: "H4 $title : ( U $author  | B $publisher ) +",
-      //ex: "H4 $title : ( U $author | U | B $publisher ) +",
-      fail: "Should not add the plain U tag. CP-3412 ",
     },
-/*    
-    "<div>" + // div 3 - test for CP-3412, optimized or-bags
-      "<h4>Title</h4>" +
-      "<U>First Author </U>" +
-      "<U>Second Author</U>" +
-      "<B>Publisher</B>" +
-    "</div>"+
-*/    
+    { name: "Orbag-7",
+      // Another test case for CP-3412,
+      // When multiple U's, the one without a $var should be last
+      targets: [
+        { p:"//body/div[3]", group:1, parent:0,
+                   type:"|", cardinality:"+" },
+        { p:"//body/div[3]/H4", group:1, v: "title" },
+        { p:"//body/div[3]/U[2]", group:1, v: "author" },
+        { p:"//body/div[3]/B", group:1, v: "publisher" },
+      ],
+      ex: "( H4 $title | U $author  | B $publisher | U ) +",
+    },
+    { name: "Orbag-8",
+      // Test case for CP-3412, should not get confused about the second U
+      targets: [
+        { p:"//body/div[4]", group:1, parent:0,
+                   type:"|", cardinality:"+" },
+        { p:"//body/div[4]/TR/TD/U", group:1, v: "author" },
+        { p:"//body/div[4]/TR/TD/B", group:1, v: "publisher" },
+      ],
+      ex: "( TD { U $author } | TD { B $publisher } | TD ) +",
+    },
+    { name: "Orbagprefix",
+      // Also CP-3412, No BR before the or-bag
+      targets: [
+        { p:"//body/div[1]/A", v: "title" },
+        { p:"//body/div[1]", group:1, parent:0,
+                   type:"|", cardinality:"+" },
+        { p:"//body/div[1]/U", group:1, v: "author" },
+        { p:"//body/div[1]/B", group:1, v: "publisher" },
+      ],
+      ex: "A $title : ( U $author | B $publisher | br ) +",
+    },
+    { name: "Orbagsuffix",
+      // Also CP-3412, No BR after the or-bag
+      targets: [
+        { p:"//body/div[1]", group:1, parent:0,
+                   type:"|", cardinality:"+" },
+        { p:"//body/div[1]/U", group:1, v: "author" },
+        { p:"//body/div[1]/B", group:1, v: "publisher" },
+        { p:"//body/div[1]/I", v: "description" },
+      ],
+      //ex: "( U $author | B $publisher | br ) +: br : I $description",
+      ex: "( U $author | B $publisher | br ) +: I $description",
+    },
+    { name: "Trfix-1",
+      // See that the TR gets repeatable
+      targets: [
+        { p:"//body/div[5]/I", v: "title"  },
+        { p:"//body/div[5]/table/TR[1]/TD/U", v: "author" , cardinality:"+" },
+      ],
+      ex: "I $title : table { TR + { TD { U + $author } } }",
+    },
+    { name: "Trfix-2",
+      // Not repeating here
+      targets: [
+        { p:"//body/div[5]/I", v: "title"  },
+        { p:"//body/div[5]/table/TR[1]/TD/U", v: "author" , cardinality:"?" },
+      ],
+      ex: "I $title : table { TR { TD { U ? $author } } }",
+    },
+    { name: "Trfix-3",
+      // Only the second TR should repeat since pattern continues with TRs
+      targets: [
+        { p:"//body/div[5]/I", v: "title"  },
+        { p:"//body/div[5]/table/TR[1]/TD/U", v: "author" , cardinality:"+" },
+        { p:"//body/div[5]/table/TR[3]/TD/B", v: "publisher" , cardinality:"+" },
+      ],
+      ex: "I $title : table { TR + { TD { U + $author } } : TR + { TD { B + $publisher } } }",
+    },
+    
   ]; // test array
 
+
   // A little helper to tidy the dumped xpatterns
   function cleanstr(s) {
     s = s.replace( /^\s*/,"" );  // leading
@@ -806,6 +1033,7 @@ XpatternMaker.prototype.unitTest = function ( ) {
 
   logger.debug("Unit test for XPatternMaker starting");
   var doc = xmlHelper.docFromString(testpage);
+  var knownfailures = 0;
   logger.debug("Test page: " + xmlHelper.serializexml(doc) );
   
   for ( var testno = 0; testno < tests.length; testno++) {
@@ -836,10 +1064,11 @@ XpatternMaker.prototype.unitTest = function ( ) {
         return false;
       }
     } else {  // Test known to fail
-      if ( res != ex )
+      if ( res != ex ) {
         logger.info("Test " + testno + ":" + t.name + " is a KNOWN failure: "
           + t.fail );
-      else {
+        knownfailures++;
+      } else {
         logger.info("Test " + testno + ":" + t.name +
            " OK, although it is a known failure!" );
         logger.info( t.fail );
@@ -848,6 +1077,9 @@ XpatternMaker.prototype.unitTest = function ( ) {
       }
     }
   }
-  logger.debug("Unit test for XPatternMaker OK");
+  var failmsg = "";  
+  if ( knownfailures )
+    failmsg = "(But with " + knownfailures + " known failures!)";  
+  logger.debug("Unit test for XPatternMaker OK. " + failmsg);
   return true; // all ok
 };
