@@ -55,8 +55,7 @@ do
       echo $QRY  >> $LOG
      
       zoomsh "open localhost:9070/${CONNECTOR}${proxyip}" \
-		"search $QRY" \
-		"show 0 1" "show 1 1" "quit"  |
+		"set count 2" "search $QRY" "quit"  |
          sed 's/localhost:9070/unix:socket/g' >>$LOG
       test $? -ne 0 && (
          echo zoomshell failed with $?
