@@ -21,7 +21,6 @@
 #include "cf_display.h"
 #include <yaz/nmem.h>
 #include <pthread.h>
-#include <X11/Xlib.h>
 
 #define MAX_DISPLAY 1000
 static pthread_mutex_t static_display_mutex;
@@ -109,6 +108,24 @@ bool CF_Display::lock(int log_fd)
         return true;
 }
 
+static int CheckDisplay(const char *cstr)
+{
+    struct stat sbuf;
+    char fname[80];
+    char *dst;
+
+    strcpy(fname, "/tmp/.X11-unix/X");
+    dst = fname + strlen(fname);
+    if (*cstr == ':')
+        cstr++;
+    while (*cstr && *cstr != '.')
+        *dst++ = *cstr++;
+    *dst = '\0';
+    if (stat(fname, &sbuf) == 0)
+        return 1;
+    return 0;
+}
+
 bool CF_Display::lock_via_xvfb_cmd(const char *lockdir, const char *xvfb_cmd,
                                    int display_base, int log_fd)
 {
@@ -325,17 +342,14 @@ bool CF_Display::lock_via_xvfb_cmd(const char *lockdir, const char *xvfb_cmd,
     int i;
     for (i = 0; i < 40; i++)
     {
-        Display *x11_display = XOpenDisplay(wrbuf_cstr(m_display));
-        if (x11_display)
+        if (CheckDisplay(wrbuf_cstr(m_display)))
         {
             if (m_logger)
                 m_logger->printf("engine", YLOG_LOG,
-                                 "display: XOpenDisplay OK for %s i=%d",
+                                 "display: CheckDisplay OK for %s i=%d",
                                  wrbuf_cstr(m_display), i);
-            XCloseDisplay(x11_display);
             return true;
         }
-
         if (m_logger && i > 10)
             m_logger->printf("engine", YLOG_WARN,
                              "display: XOpenDisplay FAIL for %s i=%d",
