@@ -22,6 +22,7 @@
 #include "type7_sort.h"
 
 #include "rpn_to_json.h"
+#include "json.h"
 
 /** Query mapping */
 struct querymap {
@@ -837,7 +838,8 @@ bool CF_RPN_to_JSON::full_rpnstructure(const Z_RPNStructure *zs)
 // used in the fullquery parameter. Get the result with get_full
 bool CF_RPN_to_JSON::parse_full(Z_RPNQuery *q,
                                 const char *sub_database,
-                                struct sort_elem *sort_list)
+                                struct sort_elem *sort_list,
+                                Z_FacetList *facet_list)
 {
     wrbuf_rewind(diag_w);
     wrbuf_rewind(full_w);
@@ -860,6 +862,11 @@ bool CF_RPN_to_JSON::parse_full(Z_RPNQuery *q,
         write_sortkey(full_w, sort_list);
         wrbuf_puts(full_w, "\"");
     }
+    if (facet_list)
+    {
+        wrbuf_puts(full_w, ",\"facets\":");
+        facet_to_json(facet_list, full_w);
+    }
     wrbuf_puts(full_w, "}");
     return true;
 }
